@@ -1,3 +1,71 @@
+/*
+ * Copyright (c) 2005-2008, 2010 Genome Research Ltd.
+ * Author(s): James Bonfield
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions are met:
+ * 
+ *    1. Redistributions of source code must retain the above copyright notice,
+ *       this list of conditions and the following disclaimer.
+ * 
+ *    2. Redistributions in binary form must reproduce the above
+ *       copyright notice, this list of conditions and the following
+ *       disclaimer in the documentation and/or other materials provided
+ *       with the distribution.
+ * 
+ *    3. Neither the names Genome Research Ltd and Wellcome Trust Sanger
+ *    Institute nor the names of its contributors may be used to endorse
+ *    or promote products derived from this software without specific
+ *    prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY GENOME RESEARCH LTD AND CONTRIBUTORS "AS
+ * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
+ * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+ * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GENOME RESEARCH
+ * LTD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+/*
+ * Author(s): James Bonfield, Simon Dear, Rodger Staden,
+ * 
+ * Copyright (c) 1994-1998, 2000-2001 MEDICAL RESEARCH COUNCIL
+ * All rights reserved
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions are met:
+ * 
+ *    1 Redistributions of source code must retain the above copyright notice, 
+ *      this list of conditions and the following disclaimer.
+ * 
+ *    2 Redistributions in binary form must reproduce the above copyright
+ *      notice, this list of conditions and the following disclaimer in
+ *      the documentation and/or other materials provided with the
+ *      distribution.
+ * 
+ *    3 Neither the name of the MEDICAL RESEARCH COUNCIL, THE LABORATORY OF
+ *      MOLECULAR BIOLOGY nor the names of its contributors may be used
+ *      to endorse or promote products derived from this software without
+ *      specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
+
 /*
  * Copyright (c) Medical Research Council 1994. All rights reserved.
  *
@@ -59,9 +127,6 @@
 #ifdef IOLIB_ZTR
 # include "io_lib/ztr.h"
 #endif
-#ifdef IOLIB_CTF
-# include "io_lib/seqIOCTF.h"
-#endif
 #ifdef IOLIB_SFF
 # include "io_lib/sff.h"
 #endif
@@ -195,12 +260,6 @@ Read *mfread_reading(mFILE *fp, char *fn, int format) {
     }
 #endif
 
-#ifdef IOLIB_CTF
-    case TT_CTF:
-	read = mfread_ctf(fp);
-	break;
-#endif
-
 #ifdef IOLIB_SFF
     case TT_SFF:
 	read = mfread_sff(fp);
@@ -340,12 +399,6 @@ int mfwrite_reading(mFILE *fp, Read *read, int format) {
     }
 #endif
 
-#ifdef IOLIB_CTF
-    case TT_CTF:
-	r = mfwrite_ctf(fp, read); 
-	break;
-#endif
-
 #ifdef IOLIB_ABI
     case TT_ABI:
 	/*return mfwrite_abi(fp, read); */
@@ -454,16 +507,6 @@ int fwrite_alf(FILE *fp, Read *read) {
 }
 #endif
 
-#ifdef IOLIB_CTF
-Read *fread_ctf(FILE *fp) {
-    return fread_reading(fp, NULL, TT_CTF);
-}
-
-int fwrite_ctf(FILE *fp, Read *read) {
-    return fwrite_reading(fp, read, TT_CTF);
-}
-#endif
-
 #ifdef IOLIB_PLN
 Read *fread_pln(FILE *fp) {
     return fread_reading(fp, NULL, TT_PLN);
