@@ -341,6 +341,10 @@ static char *cram_extract_block(cram_block *b, int size) {
  * ---------------------------------------------------------------------------
  * EXTERNAL
  */
+static void cram_external_decode_reset(cram_codec *c) {
+    c->external.b = NULL;
+}
+
 int cram_external_decode_int(cram_slice *slice, cram_codec *c,
 			     cram_block *in, char *out, int *out_size) {
     int l;
@@ -442,6 +446,7 @@ cram_codec *cram_external_decode_init(char *data, int size,
 
     c->external.type = option;
     c->external.b = NULL;
+    c->reset = cram_external_decode_reset;
 
     return c;
 }
@@ -514,6 +519,8 @@ cram_codec *cram_external_encode_init(cram_stats *st,
  * ---------------------------------------------------------------------------
  * BETA
  */
+void cram_nop_decode_reset(cram_codec *c) {}
+
 int cram_beta_decode_int(cram_slice *slice, cram_codec *c, cram_block *in, char *out, int *out_size) {
     int32_t *out_i = (int32_t *)out;
     int i, n = *out_size;
@@ -585,6 +592,8 @@ cram_codec *cram_beta_decode_init(char *data, int size,
 	return NULL;
     }
 
+    c->reset = cram_nop_decode_reset;
+
     return c;
 }
 
@@ -775,6 +784,8 @@ cram_codec *cram_subexp_decode_init(char *data, int size,
 	return NULL;
     }
 
+    c->reset = cram_nop_decode_reset;
+
     return c;
 }
 
@@ -837,6 +848,8 @@ cram_codec *cram_gamma_decode_init(char *data, int size,
 	return NULL;
     }
 
+    c->reset = cram_nop_decode_reset;
+
     return c;
 }
 
@@ -1015,6 +1028,8 @@ cram_codec *cram_huffman_decode_init(char *data, int size,
 	return NULL;
     }
 
+    h->reset = cram_nop_decode_reset;
+
     if (ncodes == 0) {
 	/* NULL huffman stream.  Ensure it returns an error if
            anything tries to use it. */
@@ -1440,6 +1455,11 @@ void cram_byte_array_len_decode_free(cram_codec *c) {
     free(c);
 }
 
+static void cram_byte_array_len_decode_reset(cram_codec *c) {
+    c->byte_array_len.len_codec->reset(c->byte_array_len.len_codec);
+    c->byte_array_len.value_codec->reset(c->byte_array_len.value_codec);
+}
+
 cram_codec *cram_byte_array_len_decode_init(char *data, int size,
 					    enum cram_external_type option,
 					    int version) {
@@ -1480,6 +1500,8 @@ cram_codec *cram_byte_array_len_decode_init(char *data, int size,
     if (cp - data != size)
         goto malformed;
 
+    c->reset = cram_byte_array_len_decode_reset;
+
     return c;
 
  malformed:
@@ -1578,6 +1600,10 @@ cram_codec *cram_byte_array_len_encode_init(cram_stats *st,
  * ---------------------------------------------------------------------------
  * BYTE_ARRAY_STOP
  */
+static void cram_byte_array_stop_decode_reset(cram_codec *c) {
+    c->byte_array_stop.b = NULL;
+}
+
 static int cram_byte_array_stop_decode_char(cram_slice *slice, cram_codec *c,
 					    cram_block *in, char *out,
 					    int *out_size) {
@@ -1694,6 +1720,7 @@ cram_codec *cram_byte_array_stop_decode_init(char *data, int size,
     }
 
     c->byte_array_stop.b = NULL;
+    c->reset = cram_byte_array_stop_decode_reset;
 
     return c;
 }
