@@ -418,4 +418,11 @@ typedef int4 int_fl;		/* f_implicit */
 }
 #endif
 
+/*-----------------------------------------------------------------------------
+ * define PATH_MAX in case it is not available (for example for hurd_i386)
+ */
+#ifndef PATH_MAX
+#    define PATH_MAX 4096
+#endif
+
 #endif /*_OS_H_*/
